#include "rtc_pwm.h"
#include "nrf.h"
#include "nrf_gpio.h"

static uint32_t m_on_time_ticks, m_off_time_ticks;

void rtc_pwm_init(void)
{
    PWM_RTC->PRESCALER = 0;
    PWM_RTC->EVTENSET = RTC_EVTENSET_COMPARE0_Msk | RTC_EVTENSET_COMPARE1_Msk;
    PWM_RTC->INTENSET = RTC_INTENSET_COMPARE0_Msk | RTC_INTENSET_COMPARE1_Msk;
    NVIC_SetPriority(PWM_RTC_IRQn, 7);
    NVIC_EnableIRQ(PWM_RTC_IRQn);

    nrf_gpio_cfg_output(RTC_PWM_PIN);
    nrf_gpio_pin_clear(RTC_PWM_PIN);
}

void rtc_pwm_set_on_off_time(uint32_t on_time_us, uint32_t off_time_us)
{
    PWM_RTC->INTENCLR = RTC_INTENCLR_COMPARE1_Msk;
    m_on_time_ticks = (uint64_t)on_time_us * RTC_TIMER_FREQ / 1000000;
    m_off_time_ticks = (uint64_t)off_time_us * RTC_TIMER_FREQ / 1000000;
    PWM_RTC->INTENSET = RTC_INTENSET_COMPARE1_Msk;
}

void rtc_pwm_start(void)
{
    PWM_RTC->CC[0] = m_on_time_ticks;
    PWM_RTC->CC[1] = m_off_time_ticks;
    PWM_RTC->TASKS_START = 1;
}

void rtc_pwm_stop(void)
{
    PWM_RTC->TASKS_STOP = 1;
    PWM_RTC->TASKS_CLEAR = 1;
}

void PWM_RTC_IRQHandler(void)
{
    if(PWM_RTC->EVENTS_COMPARE[0])
    {
        PWM_RTC->EVENTS_COMPARE[0] = 0;

        nrf_gpio_pin_clear(RTC_PWM_PIN);
    }
    if(PWM_RTC->EVENTS_COMPARE[1])
    {
        PWM_RTC->EVENTS_COMPARE[1] = 0;

        nrf_gpio_pin_set(RTC_PWM_PIN);

        PWM_RTC->CC[0] = PWM_RTC->CC[1] + m_on_time_ticks;
        PWM_RTC->CC[1] = PWM_RTC->CC[1] + m_on_time_ticks + m_off_time_ticks;
    }
}

